==> mule-metadata-model-xml-1.0.0-sources.jar: org/mule/metadata/xml/api/utils/SchemaHelper.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.metadata.xml.api.utils;

import static org.mule.apache.xerces.xs.XSSimpleTypeDefinition.FACET_ENUMERATION;
import static org.mule.apache.xerces.xs.XSSimpleTypeDefinition.FACET_PATTERN;

import org.mule.apache.xerces.impl.dv.xs.XSSimpleTypeDecl;
import org.mule.apache.xerces.xs.XSAttributeUse;
import org.mule.apache.xerces.xs.XSElementDeclaration;
import org.mule.apache.xerces.xs.XSFacet;
import org.mule.apache.xerces.xs.XSMultiValueFacet;
import org.mule.apache.xerces.xs.XSObject;
import org.mule.apache.xerces.xs.XSObjectList;
import org.mule.metadata.xml.api.schema.SchemaFetcher;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import org.apache.xmlbeans.impl.inst2xsd.Inst2Xsd;
import org.apache.xmlbeans.impl.inst2xsd.Inst2XsdOptions;
import org.apache.xmlbeans.impl.xb.xsdschema.SchemaDocument;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class SchemaHelper {

  public static Optional<String> getDefaultValue(XSElementDeclaration element) {
    return Optional.ofNullable(element.getConstraintValue());
  }

  public static Optional<String> getDefaultValue(XSAttributeUse attributeDeclaration) {
    return Optional.ofNullable(attributeDeclaration.getConstraintValue());
  }

  public static Collection<XmlDoc> getDocumentation(String annotationContent) {

    final List<XmlDoc> result = new ArrayList<>();
    final DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
    try {
      final DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
      final Document doc = dBuilder.parse(new ByteArrayInputStream(annotationContent.getBytes("UTF-8")));
      final NodeList children = doc.getDocumentElement().getChildNodes();
      for (int i = 0; i < children.getLength(); i++) {
        final Node child = children.item(i);
        if (isDocumentationTag(child)) {
          final String textContent = child.getTextContent();
          final NamedNodeMap attributes = child.getAttributes();
          final Optional<String> lang =
              Optional.ofNullable(attributes.getNamedItem("xml:lang")).map((attr) -> attr.getNodeValue());
          result.add(new XmlDoc(lang, textContent.trim()));
        }
      }
    } catch (Exception e) {
      // todo: Ignore this exception???
    }
    return result;
  }

  public static boolean isDocumentationTag(Node child) {
    return child instanceof Element
        && getLocalName((Element) child).endsWith("documentation");
  }

  public static String getLocalName(Element child) {
    final String tagName = child.getTagName();
    return tagName.contains(":") ? tagName.split(":")[1] : tagName;
  }

  public static Integer getIntegerValue(XSObject patternFacetObject) {
    final XSFacet minLengthFacet = (XSFacet) patternFacetObject;
    final String lexicalFacetValues = minLengthFacet.getLexicalFacetValue();
    if (lexicalFacetValues != null && !lexicalFacetValues.isEmpty()) {
      return Integer.parseInt(lexicalFacetValues);
    }
    return -1;
  }

  public static Double getDoubleValue(XSObject patternFacetObject) {
    final XSFacet minLengthFacet = (XSFacet) patternFacetObject;
    final String lexicalFacetValues = minLengthFacet.getLexicalFacetValue();
    if (lexicalFacetValues != null && !lexicalFacetValues.isEmpty()) {
      return Double.parseDouble(lexicalFacetValues);
    }
    return 1D;
  }

  public static List<String> generateXSD(String xmlExample) throws XmlException, IOException {
    return generateXSD(XmlObject.Factory.parse(xmlExample));
  }

  public static List<String> generateXSD(File xmlFile) throws XmlException, IOException {
    return generateXSD(XmlObject.Factory.parse(xmlFile));
  }

  private static List<String> generateXSD(XmlObject xmlInstance) throws XmlException, IOException {
    List<String> schemas = new ArrayList<>();
    Inst2XsdOptions options = new Inst2XsdOptions();
    SchemaDocument[] inst2xsd = Inst2Xsd.inst2xsd(new XmlObject[] {xmlInstance}, options);

    for (SchemaDocument schemaDocument : inst2xsd) {
      Writer writer = new StringWriter();
      XmlOptions xmlOptions = new XmlOptions();
      schemaDocument.save(writer, xmlOptions.setSavePrettyPrint());
      schemas.add(writer.toString());
    }
    return schemas;
  }

  public static XSObject getFacet(XSSimpleTypeDecl element, int facetType) {
    if (facetType == FACET_ENUMERATION || facetType == FACET_PATTERN) {
      XSObjectList list = element.getMultiValueFacets();
      for (int i = 0; i < list.getLength(); i++) {
        XSMultiValueFacet f = (XSMultiValueFacet) list.item(i);
        if (f.getFacetKind() == facetType) {
          return f;
        }
      }
    } else {
      XSObjectList list = element.getFacets();
      for (int i = 0; i < list.getLength(); i++) {
        XSFacet f = (XSFacet) list.item(i);
        if (f.getFacetKind() == facetType) {
          return f;
        }
      }
    }
    return null;
  }

  public static List<SchemaFetcher.XsdDependency> getDependencies(File schema) {
    List<SchemaFetcher.XsdDependency> dependencies = Collections.emptyList();
    try {
      dependencies = new SchemaFetcher().fetchAll(schema.toURI().toURL());
    } catch (MalformedURLException e) {
      throw new RuntimeException(e);
    }
    return dependencies;
  }

  public static class XmlDoc {

    private String content;
    private Optional<String> lang;

    public XmlDoc(Optional<String> lang, String content) {
      this.lang = lang;
      this.content = content;
    }

    public Optional<String> getLang() {
      return lang;
    }

    public String getContent() {
      return content;
    }
  }
}

==> mule-metadata-model-xml-1.1.6-sources.jar: org/mule/metadata/xml/api/utils/SchemaHelper.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.metadata.xml.api.utils;

import org.mule.metadata.xml.api.schema.SchemaFetcher;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import org.apache.xmlbeans.impl.inst2xsd.Inst2Xsd;
import org.apache.xmlbeans.impl.inst2xsd.Inst2XsdOptions;
import org.apache.xmlbeans.impl.xb.xsdschema.SchemaDocument;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class SchemaHelper {

  public static Collection<XmlDoc> getDocumentation(String annotationContent) {

    final List<XmlDoc> result = new ArrayList<>();
    final DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
    try {
      final DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
      final Document doc = dBuilder.parse(new ByteArrayInputStream(annotationContent.getBytes("UTF-8")));
      final NodeList children = doc.getDocumentElement().getChildNodes();
      for (int i = 0; i < children.getLength(); i++) {
        final Node child = children.item(i);
        if (isDocumentationTag(child)) {
          final String textContent = child.getTextContent();
          final NamedNodeMap attributes = child.getAttributes();
          final Optional<String> lang =
              Optional.ofNullable(attributes.getNamedItem("xml:lang")).map((attr) -> attr.getNodeValue());
          result.add(new XmlDoc(lang, textContent.trim()));
        }
      }
    } catch (Exception e) {
      // todo: Ignore this exception???
    }
    return result;
  }

  public static boolean isDocumentationTag(Node child) {
    return child instanceof Element
        && getLocalName((Element) child).endsWith("documentation");
  }

  public static String getLocalName(Element child) {
    final String tagName = child.getTagName();
    return tagName.contains(":") ? tagName.split(":")[1] : tagName;
  }

  public static List<SchemaFetcher.XsdDependency> getDependencies(File schema) {
    List<SchemaFetcher.XsdDependency> dependencies = Collections.emptyList();
    try {
      dependencies = new SchemaFetcher().fetchAll(schema.toURI().toURL());
    } catch (MalformedURLException e) {
      throw new RuntimeException(e);
    }
    return dependencies;
  }

  public static List<String> generateXSD(String xmlExample) throws IOException {
    try {
      return generateXSD(XmlObject.Factory.parse(xmlExample));
    } catch (XmlException e) {
      throw new IOException(e);
    }
  }

  public static List<String> generateXSD(File xmlFile) throws IOException {
    try {
      return generateXSD(XmlObject.Factory.parse(xmlFile));
    } catch (XmlException e) {
      throw new IOException(e);
    }
  }

  private static List<String> generateXSD(XmlObject xmlInstance) throws IOException {
    List<String> schemas = new ArrayList<>();
    Inst2XsdOptions options = new Inst2XsdOptions();
    SchemaDocument[] inst2xsd = Inst2Xsd.inst2xsd(new XmlObject[] {xmlInstance}, options);

    for (SchemaDocument schemaDocument : inst2xsd) {
      Writer writer = new StringWriter();
      XmlOptions xmlOptions = new XmlOptions();
      schemaDocument.save(writer, xmlOptions.setSavePrettyPrint());
      schemas.add(writer.toString());
    }
    return schemas;
  }

  public static class XmlDoc {

    private String content;
    private Optional<String> lang;

    public XmlDoc(Optional<String> lang, String content) {
      this.lang = lang;
      this.content = content;
    }

    public Optional<String> getLang() {
      return lang;
    }

    public String getContent() {
      return content;
    }
  }
}
